@@ -41,6 +41,9 @@ INVALID_ID_STRINGS = (
 	"N/A", "n/a",
 	"12345", "54321", "202020",
 	"Chassis", "chassis",
+	"Not Applicable",
+	"None",
+	"01010101-0101-0101-0101-010101010101",
 	"00020003-0004-0005-0006-000700080009",
 	"00000000",
 )
@@ -80,6 +83,9 @@ class System(object):
 		self.cpu = cpu.CPU()
 		self.hypervisor = hypervisor.Hypervisor()
 
+		# Read /proc/cpuinfo for vendor information.
+		self.__cpuinfo = self.cpu.read_cpuinfo()
+
 	def profile(self):
 		p = {}
 		p["system"] = {
@@ -270,7 +276,7 @@ class System(object):
 		"""
 			Return the system release string.
 		"""
-		return read_from_file("/etc/system-release")
+		return read_from_file("/etc/system-release") or "unknown"
 
 	@property
 	def bios_vendor(self):
@@ -279,6 +285,19 @@ class System(object):
 		"""
 		return read_from_file("/sys/class/dmi/id/bios_vendor")
 
+	def vendor_model_tuple(self):
+		try:
+			s = self.__cpuinfo["Hardware"]
+		except KeyError:
+			return (None, None)
+
+		if s.startswith("ARM-Versatile"):
+			return ("ARM", s)
+
+		v, m = s.split(maxsplit=1)
+
+		return v, m
+
 	@property
 	def vendor(self):
 		"""
@@ -290,6 +309,10 @@ class System(object):
 			if ret:
 				break
 
+		if ret is None:
+			v, m = self.vendor_model_tuple()
+			ret = v
+
 		return ret
 
 	@property
@@ -303,6 +326,10 @@ class System(object):
 			if ret:
 				break
 
+		if ret is None:
+			v, m = self.vendor_model_tuple()
+			ret = m
+
 		return ret
 
 	@property
